/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.client;

import org.jbpm.PvmException;
import org.jbpm.env.Environment;
import org.jbpm.env.session.PvmDbSession;
import org.jbpm.pvm.ProcessDefinition;


/**
 * @author Tom Baeyens
 */
public class Deploy implements Command {

  private static final long serialVersionUID = 1L;
  
  protected ProcessDefinition processDefinition;
  
  public Deploy(ProcessDefinition processDefinition) {
    this.processDefinition = processDefinition;
  }

  public Object execute(Environment environment) throws Exception {
    PvmDbSession pvmDbSession = environment.get(PvmDbSession.class);
    
    String name = processDefinition.getName();
    if (name==null) {
      throw new PvmException("process must have a name to deploy it");
    }
    
    int version = processDefinition.getVersion();
    
    if (pvmDbSession.findProcessDefinition(name, version)!=null) {
      throw new PvmException("process ["+name+"|"+version+"] already exists");
    }
    
    pvmDbSession.save(processDefinition);

    return processDefinition;
  }
}
