/*
 * JBoss, Home of Professional Open Source
 * Copyright 2005, JBoss Inc., and individual contributors as indicated
 * by the @authors tag. See the copyright.txt in the distribution for a
 * full listing of individual contributors.
 *
 * This is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as
 * published by the Free Software Foundation; either version 2.1 of
 * the License, or (at your option) any later version.
 *
 * This software is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this software; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301 USA, or see the FSF site: http://www.fsf.org.
 */
package org.jbpm.env.session;

import java.util.Collection;

import org.jbpm.pvm.Activity;
import org.jbpm.pvm.impl.ObjectReference;

/**
 * @author Tom Baeyens, Pascal Verdage
 */
public interface TimerSession {

	/**
	 * Schedule the execution of a timer.
	 * @param timer the timer to be executed.
	 * @throws IllegalArgumentException if the timer is null
	 * or if its activity is null or if its eligibleDate is null
	 * or if its eligibleDate is negative.
	 */
	public void schedule(Timer timer);

	/**
	 * Cancels an activity.
   * @param timer the timer to be cancelled
	 * @return true if the activity was "really" canceled;
	 * false if the activity was not scheduled or was already canceled
	 * or wouldn't have been executed anymore.
	 * <br />
	 * Practically, returns true if it prevents the corresponding task to be executed.
	 */
	public boolean cancel(Timer timer);
	
	/**
	 * @return the scheduled activities, i.e. the activities that will be executed.
	 */
	public Collection<ObjectReference<Activity>> getScheduledActivities();
	
	/**
	 * @return the remaining time before the next execution of an activity
	 * or -1 if there is no scheduled activity.
	 */
	public long getNextScheduledActivityTime();

	/**
	 * @return the next activity to be executed or null if none
	 */
	public ObjectReference<Activity> getNextScheduledActivity();

	/**
	 * retrieve a task that still exists from its identifier.
	 * @param id the identifier of the task to retrieve
	 * @return the corresponding task or null if none exists.
	 */
	public ObjectReference<Activity> getScheduledActivity(long id);

	/**
	 * This method is to be called after a crash.
	 * It restarts timers if needed and if they were persisted.
	 */
	public void initialize();
}
